unit EllipseBevel;

interface

uses
  Classes, Controls, ExtCtrls, Windows, Graphics, Messages, Forms;

type
  TEllipseBevel = class(TGraphicControl)
  private
    hRegion: THandle;
    FAntiAliased: Boolean;
    FStyle: TBevelStyle;
    FBevelWidth: TBevelWidth;
    procedure SetStyle(Value: TBevelStyle);
    procedure SetBevelWidth(Value: TBevelWidth);
    procedure SetAntiAliased(Value: Boolean);
  protected
    procedure WndProc(var Message: TMessage); override;
    procedure Paint; override;
  public
    procedure SetBounds(ALeft, ATop, AWidth, AHeight: Integer); override;
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  published
    property Align;
    property AntiAliased: Boolean read FAntiAliased write SetAntiAliased;
    property BevelWidth: TBevelWidth read FBevelWidth write SetBevelWidth;
    property ParentShowHint;
    property ShowHint;
    property Style: TBevelStyle read FStyle write SetStyle default bsRaised;
    property Visible;
  end;

procedure Register;

implementation

uses
  SysUtils;

const
  ColorAmountBrighten = 72;
  ColorAmountDarken = 72;

//******************************************************************
//*********************    Utility functions   *********************
//******************************************************************
function MyRGB(R, G, B: Integer): TColor;
begin
  if R < 0 then
    R := 0;
  if R > 255 then
    R := 255;
  if G < 0 then
    G := 0;
  if G > 255 then
    G := 255;
  if B < 0 then
    B := 0;
  if B > 255 then
    B := 255;
  Result := RGB(R, G, B);
end;

function BrightenColor(Color: TColor; Amount: Integer): TColor;
var
  BaseColor: LongInt;
  
begin
  BaseColor := ColorToRGB(Color);
  Result := MyRGB(GetRValue(BaseColor) + Amount, GetGValue(BaseColor) + Amount, GetBValue(BaseColor) + Amount);
end;

function DarkenColor(Color: TColor; Amount: Integer): TColor;
begin
  Result := BrightenColor(Color, -Amount);
end;

procedure SwapInt(var a, b: Integer);
var
  c: Integer;
  
begin
  c := a;
  a := b;
  b := c;
end;

function Max(a, b: Integer): Integer;
begin
  if a > b then
    Result := a
  else
    Result := b;
end;

function Min(a, b: Integer): Integer;
begin
  if a < b then
    Result := a
  else
    Result := b;
end;

procedure AntiAliasRect(B: TBitmap; X1, Y1, X2, Y2: Integer);
var
  x, y: Integer;
  p0, p1, p2: pByteArray;

begin
   if X2 < X1 then
     SwapInt(X2, X1);
   if Y2 < Y1 then
     SwapInt(Y2, Y1);
   X1 := Max(1, X1);
   Y1 := Max(1, Y1);
   X2 := Min(B.Width - 2, X2);
   Y2 := Min(B.Height - 2, Y2);
   B.PixelFormat := pf24bit;
   for y := Y1 to Y2 do
   begin
    p0 := B.ScanLine[y - 1];
    p1 := B.scanline[y];
    p2 := B.ScanLine[y + 1];
    for x := X1 to X2 do
    begin
      p1[x * 3] := (p0[x * 3] + p2[x * 3] + p1[(x - 1) * 3] + p1[(x + 1) * 3]) div 4;
      p1[x * 3 + 1] := (p0[x * 3 + 1] + p2[x * 3 + 1] + p1[(x - 1) * 3 + 1] + p1[(x + 1) * 3 + 1]) div 4;
      p1[x * 3 + 2] := (p0[x * 3 + 2] + p2[x * 3 + 2] + p1[(x - 1) * 3 + 2] + p1[(x + 1) * 3 + 2]) div 4;
      end;
   end;
end;

procedure AntiAlias(B: TBitmap);
begin
  AntiAliasRect(B, 0, 0, B.Width, B.Height);
end;

//******************************************************************
//***********************    TEllipseBevel    ***********************
//******************************************************************
constructor TEllipseBevel.Create(AOwner: TComponent);
begin
  hRegion := CreateEllipticRgn(0, 0, Width, Height);
  inherited Create(AOwner);
  ControlStyle := ControlStyle + [csReplicatable];
  FStyle := bsRaised;
  Width := 50;
  Height := 50;
  FBevelWidth := 1;
end;

destructor TEllipseBevel.Destroy;
begin
  inherited;
  DeleteObject(hRegion);
end;

procedure TEllipseBevel.SetStyle(Value: TBevelStyle);
begin
  if Value <> FStyle then
  begin
    FStyle := Value;
    Invalidate;
  end;
end;

procedure TEllipseBevel.SetBevelWidth(Value: TBevelWidth);
begin
  if Value <> FBevelWidth then
  begin
    FBevelWidth := Value;
    Invalidate;
  end;
end;

procedure TEllipseBevel.SetAntiAliased(Value: Boolean);
begin
  if Value <> FAntiAliased then
  begin
    FAntiAliased := Value;
    Invalidate;
  end;
end;

procedure TEllipseBevel.SetBounds(ALeft, ATop, AWidth, AHeight: Integer);
begin
  inherited;
  DeleteObject(hRegion);
  hRegion := CreateEllipticRgn(0, 0, Width, Height);
end;

procedure TEllipseBevel.WndProc(var Message: TMessage);
begin
  inherited;
  if (not (csDesigning in ComponentState)) and (Message.Msg = WM_MOUSEMOVE) then
    if not PtInRegion(hRegion, TWMMouse(Message).XPos, TWMMouse(Message).YPos) then
      if Application <> nil then
        Application.CancelHint;
end;

procedure TEllipseBevel.Paint;
var
  BM: TBitmap;
  R: TRect;
  i, nWidth, nHeight: Integer;

begin
  BM := TBitmap.Create;
  BM.Width := Width;
  BM.Height := Height;
  BM.Transparent := True;
  R.Left := 0;
  R.Top := 0;
  R.Right := Width;
  R.Bottom := Height;
  with BM.Canvas do
  begin
    Brush.Color := Color;
    Brush.Style := bsSolid;
    FillRect(R);
    Brush.Style := bsClear;
    Pen.Width := FBevelWidth;

    i := FBevelWidth div 2;
    nWidth := Width - i;
    nHeight := Height - i;
    if FStyle = bsLowered then
    begin
      Pen.Color := DarkenColor(Color, ColorAmountDarken);
      Arc(i, i, nWidth, nHeight, nWidth div 2 + nWidth div 4, i, i, nHeight div 2 + nHeight div 4);

      Pen.Color := DarkenColor(Color, ColorAmountDarken div 2);
      Arc(i, i, nWidth, nHeight, nWidth, i, nWidth div 2 + nWidth div 4, i);
      Arc(i, i, nWidth, nHeight, i, nHeight div 2 + nHeight div 4, i, nHeight);

      Pen.Color := BrightenColor(Color, ColorAmountBrighten);
      Arc(i, i, nWidth, nHeight, nWidth div 4, nHeight, nWidth, nHeight div 4);

      Pen.Color := BrightenColor(Color, ColorAmountBrighten div 4);
      Arc(i, i, nWidth, nHeight, nWidth, nHeight div 4, nWidth, i);
      Arc(i, i, nWidth, nHeight, i, nHeight, nWidth div 4, nHeight);

      Pen.Color := Color;
      Arc(i, i, nWidth, nHeight, i, nHeight - (nHeight div 12), nWidth div 12, nHeight);
      Arc(i, i, nWidth, nHeight, nWidth, nHeight div 12, nWidth - (nWidth div 12), i);
    end
    else
    begin
      Pen.Color := BrightenColor(Color, ColorAmountBrighten);
      Arc(i, i, nWidth, nHeight, nWidth div 2 + nWidth div 4, i, i, nHeight div 2 + nHeight div 4);

      Pen.Color := BrightenColor(Color, ColorAmountBrighten div 4);
      Arc(i, i, nWidth, nHeight, nWidth, i, nWidth div 2 + nWidth div 4, i);
      Arc(i, i, nWidth, nHeight, i, nHeight div 2 + nHeight div 4, i, nHeight);

      Pen.Color := DarkenColor(Color, ColorAmountDarken);
      Arc(i, i, nWidth, nHeight, nWidth div 4, nHeight, nWidth, nHeight div 4);

      Pen.Color := DarkenColor(Color, ColorAmountDarken div 2);
      Arc(i, i, nWidth, nHeight, nWidth, nHeight div 4, nWidth, i);
      Arc(i, i, nWidth, nHeight, i, nHeight, nWidth div 4, nHeight);

      Pen.Color := Color;
      Arc(i, i, nWidth, nHeight, i, nHeight - (nHeight div 12), nWidth div 12, nHeight);
      Arc(i, i, nWidth, nHeight, nWidth, nHeight div 12, nWidth - (nWidth div 12), i);
    end;
  end;
  if FAntiAliased then
    AntiAlias(BM);
//  BitBlt(Canvas.Handle, 0, 0, Width, Height, BM.Canvas.Handle, 0, 0, SRCAND);
  Canvas.Draw(0, 0, BM);
  BM.Free;
end;

procedure Register;
begin
  RegisterComponents('SEDLAN A.D.', [TEllipseBevel]);
end;

end.

